<?php

namespace Drupal\k8s\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Render\RendererInterface;
use Drupal\k8s\Service\K8sServiceInterface;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Controller responsible for "update" urls.
 *
 * This class is mainly responsible for
 * updating the aws entities from urls.
 */
class ApiController extends ControllerBase implements ApiControllerInterface {

  /**
   * The K8s Service.
   *
   * @var \Drupal\k8s\Service\K8sServiceInterface
   */
  private $k8sService;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  private $requestStack;

  /**
   * Renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  private $renderer;

  /**
   * ApiController constructor.
   *
   * @param \Drupal\k8s\Service\K8sServiceInterface $k8s_service
   *   Object for interfacing with K8s Api.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   Messanger Object.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Request stack object.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct(K8sServiceInterface $k8s_service, Messenger $messenger, RequestStack $request_stack, RendererInterface $renderer) {
    $this->k8sService = $k8s_service;
    $this->messenger = $messenger;
    $this->requestStack = $request_stack;
    $this->renderer = $renderer;
  }

  /**
   * Dependency Injection.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('k8s'),
      $container->get('messenger'),
      $container->get('request_stack'),
      $container->get('renderer')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function updateNodeList($cloud_context) {
    $this->k8sService->setCloudContext($cloud_context);
    $updated = $this->k8sService->updateNodes();

    if ($updated != FALSE) {
      $this->messageUser($this->t('Updated Nodes.'));
    }
    else {
      $this->messageUser($this->t('Unable to update Nodes.'), 'error');
    }

    return $this->redirect('view.k8s_node.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function updateNamespaceList($cloud_context) {
    $this->k8sService->setCloudContext($cloud_context);
    $updated = $this->k8sService->updateNamespaces();

    if ($updated != FALSE) {
      $this->messageUser($this->t('Updated Namespaces.'));
    }
    else {
      $this->messageUser($this->t('Unable to update Namespaces.'), 'error');
    }

    return $this->redirect('view.k8s_namespace.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function updatePodList($cloud_context) {
    $this->k8sService->setCloudContext($cloud_context);
    $updated = $this->k8sService->updatePods();

    if ($updated != FALSE) {
      $this->messageUser($this->t('Updated Pods.'));
    }
    else {
      $this->messageUser($this->t('Unable to update Pods.'), 'error');
    }

    return $this->redirect('view.k8s_pod.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

  /**
   * Helper method to add messages for the end user.
   *
   * @param string $message
   *   The message.
   * @param string $type
   *   The message type: error or message.
   */
  private function messageUser($message, $type = 'message') {
    switch ($type) {
      case 'error':
        $this->messenger->addError($message);
        break;

      case 'message':
        $this->messenger->addMessage($message);
      default:
        break;
    }
  }

}
