<?php

namespace Drupal\k8s\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Component\Serialization\Yaml;

/**
 * Form controller for the pod entity create form.
 *
 * @ingroup k8s
 */
class PodCreateForm extends K8sContentForm {

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = '') {
    $form = parent::buildForm($form, $form_state);
    $this->k8sService->setCloudContext($cloud_context);

    $weight = -50;

    $form['pod'] = [
      '#type' => 'details',
      '#title' => $this->t('Pod'),
      '#open' => TRUE,
      '#weight' => $weight++,
    ];

    $form['pod']['namespace'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Namespace'),
      '#options'       => $this->getNamespaceOptions(),
      '#default_value' => 'default',
      '#required'      => TRUE,
    ];

    $form['pod']['detail'] = $form['detail'];
    unset($form['detail']);

    $this->addOthersFieldset($form, $weight++, $cloud_context);

    $form['actions'] = $this->actions($form, $form_state, $cloud_context);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $this->trimTextfields($form, $form_state);

    $entity = $this->entity;

    $cloud_context = $this->routeMatch->getParameter('cloud_context');
    $this->k8sService->setCloudContext($cloud_context);

    $params = Yaml::decode($entity->getDetail());
    $result = $this->k8sService->createPod($entity->getNamespace(), $params);
    if (!empty($result)
      && ($entity->setName($result['metadata']['name']))
      && ($entity->save())
    ) {

      // Update the pod.
      $this->k8sService->updatePods([
        'metadata.name' => $entity->getName(),
      ], FALSE);

      $message = $this->t('The @label "%label" has been created.', [
        '@label'       => $entity->getEntityType()->getLabel(),
        '%label'       => $entity->label(),
      ]);

      $this->messenger->addMessage($message);
      $form_state->setRedirect('view.k8s_pod.list', ['cloud_context' => $entity->getCloudContext()]);
    }
    else {
      $message = $this->t('The @label "%label" couldn\'t create.', [
        '@label' => $entity->getEntityType()->getLabel(),
        '%label' => $entity->label(),
      ]);
      $this->messenger->addError($message);
    }
  }

  /**
   * Get select options of namespace.
   *
   * @return array
   *   The options of namespace.
   */
  private function getNamespaceOptions() {
    $namespaces = $this->k8sService->getNamespaces();
    $options = [];
    foreach ($namespaces as $namespace) {
      $name = $namespace['metadata']['name'];
      $options[$name] = $name;
    }

    return $options;
  }

}
