<?php

namespace Drupal\k8s\Service;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Component\Serialization\Yaml;

use Drupal\k8s\Entity\K8sNode;
use Drupal\k8s\Entity\K8sNamespace;
use Drupal\k8s\Entity\Pod;

/**
 * Entity update methods for Batch API processing.
 */
class K8sBatchOperations {

  /**
   * The finish callback function.
   *
   * Deletes stale entities from the database.
   *
   * @param string $entity_type
   *   The entity type.
   * @param array $stale
   *   The stale entities to delete.
   * @param bool $clear
   *   TRUE to clear entities, FALSE keep them.
   */
  public static function finished($entity_type, array $stale, $clear = TRUE) {
    $entity_type_manager = \Drupal::entityTypeManager();
    if (count($stale) && $clear == TRUE) {
      $entity_type_manager->getStorage($entity_type)->delete($stale);
    }
  }

  /**
   * Update or create a k8s node entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $node
   *   The node array.
   */
  public static function updateNode($cloud_context, array $node) {
    $k8s_service = \Drupal::service('k8s');
    $k8s_service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $node['metadata']['name'];
    $entity_id = $k8s_service->getEntityId('k8s_node', 'name', $name);

    $status = '';
    $last_condition = end($node['status']['conditions']);
    if (!empty($last_condition)) {
      $status = $last_condition['type'];
    }

    if (!empty($entity_id)) {
      $entity = K8sNode::load($entity_id);
    }
    else {
      $entity = K8sNode::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'created' => strtotime($node['metadata']['creationTimestamp']),
        'changed' => strtotime($node['metadata']['creationTimestamp']),
        'refreshed' => $timestamp,
      ]);
    }

    $entity->setStatus($status);

    // Labels.
    self::setKeyValueTypeFieldValue($entity, 'labels', $node['metadata']['labels']);

    // Annotations.
    self::setKeyValueTypeFieldValue($entity, 'annotations', $node['metadata']['annotations']);

    // Addresses.
    $map = [];
    foreach ($node['status']['addresses'] as $address) {
      $map[$address['type']] = $address['address'];
    }
    self::setKeyValueTypeFieldValue($entity, 'addresses', $map);

    $entity->setPodCidr($node['spec']['podCIDR']);
    $entity->setProviderId($node['spec']['providerID']);
    $entity->setUnschedulable(isset($node['spec']['unschedulable']) ?: FALSE);
    $entity->setMachineId($node['status']['nodeInfo']['machineID']);
    $entity->setSystemUuid($node['status']['nodeInfo']['systemUUID']);
    $entity->setBootId($node['status']['nodeInfo']['bootID']);
    $entity->setKernelVersion($node['status']['nodeInfo']['kernelVersion']);
    $entity->setOsImage($node['status']['nodeInfo']['osImage']);
    $entity->setContainerRuntimeVersion($node['status']['nodeInfo']['containerRuntimeVersion']);
    $entity->setKubeletVersion($node['status']['nodeInfo']['kubeletVersion']);
    $entity->setKubeProxyVersion($node['status']['nodeInfo']['kubeProxyVersion']);
    $entity->setOperatingSystem($node['status']['nodeInfo']['operatingSystem']);
    $entity->setArchitecture($node['status']['nodeInfo']['architecture']);

    $entity->setRefreshed($timestamp);
    $entity->save();
  }

  /**
   * Update or create a k8s namespace entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $namespace
   *   The namespace array.
   */
  public static function updateNamespace($cloud_context, array $namespace) {
    $k8s_service = \Drupal::service('k8s');
    $k8s_service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $namespace['metadata']['name'];
    $entity_id = $k8s_service->getEntityId('k8s_namespace', 'name', $name);

    $status = $namespace['status']['phase'];

    if (!empty($entity_id)) {
      $entity = K8sNamespace::load($entity_id);
    }
    else {
      $entity = K8sNamespace::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'created' => strtotime($namespace['metadata']['creationTimestamp']),
        'changed' => strtotime($namespace['metadata']['creationTimestamp']),
        'refreshed' => $timestamp,
      ]);
    }

    $entity->setStatus($status);

    // Labels.
    $labels = [];
    if (!empty($namespace['metadata']['labels'])) {
      $labels = $namespace['metadata']['labels'];
    }
    self::setKeyValueTypeFieldValue($entity, 'labels', $labels);

    $entity->setRefreshed($timestamp);
    $entity->save();
  }

  /**
   * Update or create a k8s pod entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $pod
   *   The pod array.
   */
  public static function updatePod($cloud_context, array $pod) {
    $k8s_service = \Drupal::service('k8s');
    $k8s_service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $pod['metadata']['name'];
    $namespace = $pod['metadata']['namespace'];
    $entity_id = $k8s_service->getEntityId(
      'k8s_pod',
      'name',
      $name,
      ['namespace' => $namespace]
    );

    if (!empty($entity_id)) {
      $entity = Pod::load($entity_id);
    }
    else {
      $entity = Pod::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'created' => strtotime($pod['metadata']['creationTimestamp']),
        'changed' => strtotime($pod['metadata']['creationTimestamp']),
        'refreshed' => $timestamp,
      ]);
    }

    // Labels.
    self::setKeyValueTypeFieldValue(
      $entity,
      'labels',
      isset($pod['metadata']['labels']) ? $pod['metadata']['labels'] : []
    );

    // Annotations.
    self::setKeyValueTypeFieldValue(
      $entity,
      'annotations',
      isset($pod['metadata']['annotations']) ? $pod['metadata']['annotations'] : []
    );

    // Containers.
    $containers = [];
    foreach ($pod['spec']['containers'] as $container_data) {
      $containers[] = Yaml::encode($container_data);
    }
    $entity->setContainers($containers);

    // Restarts.
    if (!empty($pod['status']['containerStatuses'])) {
      $entity->setRestarts($pod['status']['containerStatuses'][0]['restartCount']);
    }

    // Detail.
    $entity->setDetail(Yaml::encode($pod));

    $entity->setNamespace($pod['metadata']['namespace']);
    $entity->setStatus($pod['status']['phase']);
    $entity->setQosClass($pod['status']['qosClass']);
    $entity->setNodeName($pod['spec']['nodeName']);
    $entity->setPodIp($pod['status']['podIP']);

    $entity->setRefreshed($timestamp);
    $entity->save();
  }

  /**
   * Set key_value type field value.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity object.
   * @param string $field_name
   *   The field name.
   * @param array $value_map
   *   The value of map type.
   */
  private static function setKeyValueTypeFieldValue(EntityInterface $entity, $field_name, array $value_map) {
    $key_values = [];
    if (!isset($value_map)) {
      $value_map = [];
    }
    foreach ($value_map as $key => $value) {
      $key_values[] = ['item_key' => $key, 'item_value' => $value];
    }

    usort($key_values, function ($a, $b) {
      return strcmp($a['item_key'], $b['item_key']);
    });

    $entity->set($field_name, $key_values);
  }

}
