<?php

namespace Drupal\k8s\Service;

/**
 * Interface K8sServiceInterface.
 */
interface K8sServiceInterface {

  /**
   * Set the cloud context.
   *
   * @param string $cloud_context
   *   Cloud context string.
   */
  public function setCloudContext($cloud_context);

  /**
   * Get pods.
   *
   * @param array $params
   *   Parameters array to send to API.
   *
   * @return mixed
   *   An array of results or NULL if there is an error.
   */
  public function getPods(array $params = []);

  /**
   * Update the Pods.
   *
   * Delete old Pod entities, query the api for updated entities and store
   * them as Pod entities.
   *
   * @param array $params
   *   Optional parameters array.
   * @param bool $clear
   *   TRUE to clear stale security groups.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updatePods(array $params = [], $clear = TRUE);

  /**
   * Create k8s pod.
   *
   * @param string $namespace
   *   The namespace of the pod.
   * @param array $params
   *   Parameters array to send to API.
   *
   * @return mixed
   *   An array of results or NULL if there is an error.
   */
  public function createPod($namespace, array $params = []);

  /**
   * Update k8s pod.
   *
   * @param string $namespace
   *   The namespace of the pod.
   * @param array $params
   *   Parameters array to send to API.
   *
   * @return mixed
   *   An array of results or NULL if there is an error.
   */
  public function updatePod($namespace, array $params = []);

  /**
   * Delete k8s pod.
   *
   * @param string $namespace
   *   The namespace of the pod.
   * @param array $params
   *   Parameters array to send to API.
   *
   * @return mixed
   *   An array of results or NULL if there is an error.
   */
  public function deletePod($namespace, array $params = []);

  /**
   * Get k8s nodes.
   *
   * @param array $params
   *   Parameters array to send to API.
   *
   * @return mixed
   *   An array of results or NULL if there is an error.
   */
  public function getNodes(array $params = []);

  /**
   * Get k8s namespaces.
   *
   * @param array $params
   *   Parameters array to send to API.
   *
   * @return mixed
   *   An array of results or NULL if there is an error.
   */
  public function getNamespaces(array $params = []);

  /**
   * Create k8s namespace.
   *
   * @param array $params
   *   Parameters array to send to API.
   *
   * @return mixed
   *   An array of results or NULL if there is an error.
   */
  public function createNamespace(array $params = []);

  /**
   * Update k8s namespace.
   *
   * @param array $params
   *   Parameters array to send to API.
   *
   * @return mixed
   *   An array of results or NULL if there is an error.
   */
  public function updateNamespace(array $params = []);

  /**
   * Delete k8s namespace.
   *
   * @param array $params
   *   Parameters array to send to API.
   *
   * @return mixed
   *   An array of results or NULL if there is an error.
   */
  public function deleteNamespace(array $params = []);

  /**
   * Update the Nodes.
   *
   * Delete old Node entities, query the api for updated entities and store
   * them as Node entities.
   *
   * @param array $params
   *   Optional parameters array.
   * @param bool $clear
   *   TRUE to clear stale security groups.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateNodes(array $params = [], $clear = TRUE);

  /**
   * Update the Namespaces.
   *
   * Delete old Namespace entities, query the api for updated entities and store
   * them as Namespace entities.
   *
   * @param array $params
   *   Optional parameters array.
   * @param bool $clear
   *   TRUE to clear stale security groups.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateNamespaces(array $params = [], $clear = TRUE);

  /**
   * Helper method to load an entity using parameters.
   *
   * @param string $entity_type
   *   Entity Type.
   * @param string $id_field
   *   Entity id field.
   * @param string $id_value
   *   Entity id value.
   * @param array $extra_conditions
   *   Extra conditions.
   *
   * @return int
   *   Entity id.
   */
  public function getEntityId($entity_type, $id_field, $id_value, array $extra_conditions = []);

}
